/*
 * Serializer: 
 */
package model;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

abstract public class Serializer
{
	//
	// 属性
	//
	private final String charsetName;
	
	//
	// コンストラクタ
	//
	protected Serializer(
		String strCharsetName)	// 文字符号化方式
	{
		if (strCharsetName != null && strCharsetName.length() == 0) {
			throw new IllegalArgumentException();
		}
		
		charsetName = strCharsetName;
	}

	//
	// 操作
	//
	
	// シリアライズする
	final public void serialize(
		OutputStream stream)	// 出力対象ストリーム
		throws IOException
	{
		if (stream == null) {
			throw new NullPointerException();
		}

		OutputStreamWriter osw = null;
		BufferedWriter writer = null;
		try {
			osw = (charsetName != null
								? new OutputStreamWriter(stream, charsetName)
								: new OutputStreamWriter(stream));
			writer = new BufferedWriter(osw);
			writeHeader(writer);
			writeBody(writer);
			writeFooter(writer);
		}
		finally {
			if (writer != null) {
				try {
					writer.close();
				}
				catch (IOException exp) {
				}
			}
			else if (osw != null) {
				try {
					osw.close();
				}
				catch (IOException exp) {
				}
			}
		}
	}

	//
	// 再定義必須メソッド(実装)
	//

	abstract protected void writeHeader(Writer writer) throws IOException;
	abstract protected void writeBody(Writer writer) throws IOException;
	abstract protected void writeFooter(Writer writer) throws IOException;
}
