/*
 * Actionfinition: 操作定義
 */

package ui;

import java.awt.Image;
import java.util.EventListener;

public final class ActionDefinition
{
	//
	// 内部型定義
	//

	// Style: 種類 
	// 『Effective Java』「項目21 enum構文をクラスで置き換える」に従って実装
	public static final class Style
	{
		//
		// クラス属性
		//

		public static final Style NORMAL	= new Style("Normal");
		public static final Style CHECK	= new Style("Check");
		public static final Style RADIO	= new Style("Radio");

		//
		// 属性
		//
		private final String name;

		//
		// コンストラクタ
		//
		private Style(
			String strName)	// 名前
		{
			 name = strName;
		}

		//
		// 再定義メソッド(状態)
		//

		// 文字列化した結果を返す
		public String toString()
		{
			return name;
		}
	}

	//
	// クラス属性
	//
	
	private static final boolean DEBUG	= false;

	//
	// 属性
	//

	public final EventListener listener;
	public final String label;
	public final Image image;
	public final Style style;

	//
	// コンストラクタ
	//
	public ActionDefinition(
		EventListener l,	// リスナー 
		String strLabel,	// 表記
		Image actionImage,	// イメージ
		Style actionStyle)	// スタイル
	{
		if (DEBUG) {
			if (l == null || strLabel == null) {
				throw new NullPointerException();
			}
			else if (strLabel.length() == 0) {
				throw new IllegalArgumentException();
			}
		}

		listener = l;
		label = strLabel;
		image = actionImage;
		style = actionStyle;
	}
	public ActionDefinition(
		EventListener l,	// リスナー 
		String strLabel,	// 表記
		Image img)			// イメージ
	{
		this(l, strLabel, img, Style.NORMAL);
	}
	public ActionDefinition(
		EventListener l,	// リスナー 
		String strLabel)	// 表記
	{
		this(l, strLabel, null, Style.NORMAL);
	}
	public ActionDefinition(
		EventListener l,	// リスナー 
		String strLabel,	// 表記
		Style actionStyle)	// スタイル
	{
		this(l, strLabel, null, actionStyle);
	}
}
