/*
 * ActionManager: AWT用アクションマネージャー
 */
package ui.awt;

import java.awt.Button;
import java.awt.Component;
import java.awt.Frame;
import java.awt.MenuBar;
import java.awt.Menu;
import java.awt.CheckboxMenuItem;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.util.EventListener;
import java.util.Map;

import ui.ActionDefinition;
import ui.MenuItemDefinition;

public final class ActionManager extends ui.ActionManager
{
	//
	// コンストラクタ
	//
	public ActionManager()
	{
		super();
	}

	//
	// 再定義メソッド(操作)
	//

	// メニューを作成する 	
	public void buildMenu(
		String strLabel,					// 表記
		int nMenuMnemonic,					// ニーモニック
		MenuItemDefinition[] definitions,	// メニュー項目定義
		Frame frameTarget)					// 作成対象フレーム
	{
		if (DEBUG) {
			if (strLabel == null) {
				throw new NullPointerException();
			}
			else if (strLabel.length() == 0) {
				throw new IllegalArgumentException();
			}
		/*	
			if (definitions == null) {
				throw new NullPointerException();
			}
			else if (definitions.length == 0) {
				throw new IllegalArgumentException();
			}
		 */
			if (frameTarget == null) {
				throw new NullPointerException();
			}
		}

		MenuBar menuBar = frameTarget.getMenuBar(); 
		if (menuBar == null) {
			menuBar = new MenuBar();
			frameTarget.setMenuBar(menuBar);
		}

		final Menu menu = new Menu(getStrictLabel(strLabel));

		// 仮の実装
		if (definitions == null) {
			menuBar.add(menu);
			return;
		}

		Map mapMenuItem = getMenuItemMap(); 
		for (int nIndex = 0; nIndex < definitions.length; nIndex++) {
			MenuItemDefinition itemDef = definitions[nIndex];
			if (itemDef == MenuItemDefinition.SEPARATOR) {
				menu.addSeparator();
			}
			else {
				ActionDefinition actionDef = itemDef.action;
				MenuItem item;
				if (actionDef.style == ActionDefinition.Style.CHECK) {
					CheckboxMenuItem checkItem = new CheckboxMenuItem(getStrictLabel(actionDef.label));
					checkItem.addItemListener((ItemListener)actionDef.listener);
					item = checkItem;
				}
				else if (actionDef.style == ActionDefinition.Style.NORMAL) {
					item = new MenuItem(getStrictLabel(actionDef.label));
					item.addActionListener((ActionListener)actionDef.listener);
				}
				else {
					throw new IllegalArgumentException();
				}
				
				if (itemDef.accelerator != KeyEvent.CHAR_UNDEFINED) {
					item.setShortcut(new MenuShortcut(itemDef.accelerator));
				}

				menu.add(item);
				
				mapMenuItem.put(actionDef.listener, item);
			}
		}

		menuBar.add(menu);
	}

	// 有効 / 無効を設定する
	public void setEnable(
		EventListener listener,	// リスナー
		boolean bEnable)			// 有効状態
	{
		if (DEBUG) {
			if (listener == null) {
				throw new NullPointerException();
			}
		}

		Map mapMenuItem = getMenuItemMap(); 
		if (mapMenuItem.containsKey(listener)) {
			MenuItem item = (MenuItem)mapMenuItem.get(listener);
			item.setEnabled(bEnable);
		}
	}

	// チェック状態を設定する
	public void setCheckStatus(
		ItemListener listener,	// リスナー
		boolean bCheck)		// チェック状態
	{
		if (DEBUG) {
			if (listener == null) {
				throw new NullPointerException();
			}
		}

		Map mapMenuItem = getMenuItemMap(); 
		if (mapMenuItem.containsKey(listener)) {
			CheckboxMenuItem checkItem = (CheckboxMenuItem)mapMenuItem.get(listener);
			checkItem.setState(bCheck);
		}
	}

	//
	// 再定義メソッド(実装)
	//

	// ツールバー用コンポーネントを作成する
	protected Component createToolBarComponent(
		ActionDefinition actionDefinition)
	{
		if (actionDefinition == null) {
			throw new NullPointerException();
		}
		else if (actionDefinition.style != ActionDefinition.Style.NORMAL) {
			// 未実装
			throw new IllegalArgumentException();
		}

		Button button; 
		if (actionDefinition.image != null) {
			button = new ImageButton(actionDefinition.image); 
		}
		else {
			button = new Button(getStrictLabel(actionDefinition.label)); 
		}
		button.addActionListener((ActionListener)actionDefinition.listener);
		return button;
	}
}
