/*
 * DialogUtilities: ダイアログ用各種メソッド
 */
package ui.awt;

import java.awt.Button;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import ui.CommonLabelManager;

public final class DialogUtilities
{
	//
	// クラスメソッド(公開)
	//

	// OKボタンを作成する
	public static Button createOkButton(
		ActionListener listener)	// アクションリスナー
	{
		if (listener == null) {
			throw new NullPointerException();
		}
		
		Button button = new Button(CommonLabelManager.getLabel(CommonLabelManager.OK));
		button.addActionListener(listener);
		return button;
	}
	
	// キャンセルボタンを作成する
	public static Button createCancelButton(
		ActionListener listener)	// アクションリスナー
	{
		if (listener == null) {
			throw new NullPointerException();
		}
		
		Button button = new Button(CommonLabelManager.getLabel(CommonLabelManager.CANCEL));
		button.addActionListener(listener);
		return button;
	}

	// 数字キー入力KeyListenerを作成する
	// 注記: Zaurus用J2MEでは効果がない
	public static KeyListener createNumberKeyListener(
		final KeyListener listener)	// 設定KeyListener
	{
		return new KeyListener() {
			public void keyTyped(
				KeyEvent e) {
				// 入力を数字のみに制限する
				if (Character.isISOControl(e.getKeyChar()) == false
				&& Character.isDigit(e.getKeyChar()) == false) {
					// Zaurus用J2MEではここでcosume()を呼び出してもキーが入力されてしまう
					// そのためOK実行時のチェックは必須
					e.consume();
					java.awt.Toolkit.getDefaultToolkit().beep();
				}
				else if (listener != null) {
					listener.keyTyped(e);
				}
			}
		
			public void keyPressed(
				KeyEvent e) {
				if (listener != null) {
					listener.keyPressed(e);
				}
			}
		
			public void keyReleased(
				KeyEvent e) {
				if (listener != null) {
					listener.keyReleased(e);
				}
			}
		};
	}
	
	// 操作ボタンを含むパネルを作成する
	public static Container createResponseButtonContainer(
		Button[] buttons)	// ボタンの集合
	{
		if (buttons == null) {
			throw new NullPointerException();
		}
		else if (buttons.length == 0) {
			throw new IllegalArgumentException();
		}

		Container panel = new Container();
		panel.setLayout(new GridLayout(1, buttons.length));
		for (int nIndex = 0; nIndex < buttons.length; nIndex++) {
			panel.add(buttons[nIndex]);
		}
		return panel;
	}

	// ウィンドウを参照ウィンドウの中央に移動する
	// 注記: Window#setLocationRelativeToメソッドの代替メソッド。
	// Java2 1.3ではJDialogで定義されていたためZaurus用J2MEには存在しない。
	public static void moveWindowToCenter(
		Window referenceWindow,	// 参照ウィンドウ(入力)
		Window targetWindow)	// 対象ウィンドウ(変更)
	{
		Rectangle ownerInsets = referenceWindow.getBounds();
		int nLeft = ownerInsets.x
				 + ((ownerInsets.width / 2)
					- (targetWindow.getWidth() / 2));
		int nTop = ownerInsets.y
				 + ((ownerInsets.height / 2)
					- (targetWindow.getHeight() / 2));
		targetWindow.setLocation(nLeft, nTop);
	}

	//
	// コンストラクタ
	//
	private DialogUtilities()
	{
	}
}
