/*
 * EquinoxHolidayInfo : 春分 / 秋分祝日クラス
 */

import HolidayInfo;

public final class EquinoxHolidayInfo extends HolidayInfo
{
	//
	// 属性
	//
	private final int	day;

	//
	// 構築子
	//

	public EquinoxHolidayInfo(
		String strCountry,	// 国(入力)
		String strName,		// 祝日名(入力)
		int nMonth,		// 月(入力)
		int nDays,			// 日数(入力)
		int nFirstYear,	// 開始年(入力)
		int nLastYear)		// 終了年(入力)
	{
		super(strCountry, strName, nMonth, nDays, nFirstYear, nLastYear);

		if (nMonth != 2 && nMonth != 8) {
			throw new IllegalArgumentException();
		}

		day = (nMonth == 2 ? 21 : 23);
	}

	//
	// 実装
	//

	// 指定年の日付を返す
	protected int getDay(
		int year)	// 日付取得対象年
	{
		int strictDay = day;

		// 春分 / 秋分の日
		// 1900年から2099年までの春分／秋分の日は次の計算式によって求めることが
		// できる :
		// 春分の日……(0.24242 * 年) - (年 / 4) + 35.84
		// 秋分の日……(0.24204 * 年) - (年 / 4) + 39.01
		// この式の出典は http://www.top.or.jp/~cpop/syunbun.htm。
		if (month() == 2) {
			// 三月
			if (1900 <= year && year <= 2099) {
				strictDay = (int)Math.floor((0.24242 * year)
									- (year / 4) + 35.84);
			}
		}
		else if (month() == 8) {
			// 九月
			if (1900 <= year && year <= 2099) {
				strictDay = (int)Math.floor((0.24204 * year)
									- (year / 4) + 39.01);
			}
		}
		return strictDay;
	}
}