/*
 * HolidayInfoList : 祝日情報リスト
 */

//import java.io.*;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

public final class HolidayInfoList
{
	//
	// クラスメソッド(公開)
	//

	// ファイルからクラスを作成する
	public static HolidayInfoList createFromFile(
		InputStream streamHolidayInfo)	// 祝日定義ファイルの入力ストリーム(入力)
		throws IOException
	{
		if (streamHolidayInfo == null) {
			throw new NullPointerException();
		}

		HolidayInfoList infoList = new HolidayInfoList();

		BufferedReader reader = null;
		try {
			// SL-C7x0 / C8x0のJ2MEは日本語ファイルの文字符号化方式としてUTF-8しか
			// 受けつけない、とのこと
		//	reader = new BufferedReader(new InputStreamReader(streamHolidayInfo, "JISAutoDetect"));
			reader = new BufferedReader(new InputStreamReader(streamHolidayInfo, "UTF-8"));
			String line;
			while ((line = reader.readLine()) != null) {
				if (line.length() == 0 || line.charAt(0) == '#') {
					continue;
				}

				List listCsvColumn = new ArrayList();
				StringTokenizer tokenizer = new StringTokenizer(line, ",");
				String token;
				while (tokenizer.hasMoreTokens()
				&& (token = tokenizer.nextToken()) != null) {
					listCsvColumn.add(token);
				}
				if (listCsvColumn.size() < 5) {
					break;
				}
				else if (listCsvColumn.size() == 5) {
					listCsvColumn.add("9999");
				}

				// カラムは順に名前,月,日またはn週m曜日,日数,開始年,終了年

				String strCountry = "JP";
				String strName = (String)listCsvColumn.get(0);
				int nMonth = Integer.parseInt((String)listCsvColumn.get(1));
				nMonth--;
				String strDayInfo = (String)listCsvColumn.get(2);
				int nDays = Integer.parseInt((String)listCsvColumn.get(3));
				int nFirstYear = Integer.parseInt((String)listCsvColumn.get(4));
				int nLastYear = Integer.parseInt((String)listCsvColumn.get(5));

				HolidayInfo info = null;
				if (strDayInfo.equals("eqx")) {
					// 春分/秋分の日
					info = new EquinoxHolidayInfo(strCountry,
													strName,
													nMonth,
													nDays,
													nFirstYear,
													nLastYear);
				}
				else if (strDayInfo.charAt(0) == 'd') {
					// 日付指定
					info = new DateHolidayInfo(strCountry,
												strName,
												nMonth,
												Integer.parseInt(strDayInfo.substring(1)),
												nDays,
												nFirstYear,
												nLastYear);
				}
				else if (strDayInfo.charAt(0) == 'w') {
					// n週m曜日指定
					int nIndex = strDayInfo.indexOf('d');
					int nWeekOfMonth = Integer.parseInt(strDayInfo.substring(1, nIndex));
					int nDayOfWeek = Integer.parseInt(strDayInfo.substring(nIndex + 1));
					info = new WomDowHolidayInfo(strCountry,
												strName,
												nMonth,
												nWeekOfMonth,
												nDayOfWeek,
												nDays,
												nFirstYear,
												nLastYear);
				}

				if (info != null) {
					infoList.infoList.add(info);
				}
			}
		}
		catch (IOException exp) {
			throw exp;
		}
		finally {
			try {
				if (reader != null) {
					reader.close();
				}
			}
			catch (Exception exp) {
				// Do nothing.
			}
		}

		return infoList;
	}

	//
	// 属性
	//
	private List infoList = new ArrayList();

	//
	// 構築子
	//
	public HolidayInfoList()
	{
	}

	//
	// 状態
	//

	// 指定年の祝日一覧を返す
	public List getHolidaysOfYear(
		int nYear) // 祝日一覧を取得する年(入力)
	{
		// まず指定年の休日一覧を取得する
		List holidays = new ArrayList();
		Iterator it = infoList.iterator();
		while (it.hasNext()) {
			HolidayInfo.DateOfYear holiday = ((HolidayInfo)it.next()).getDateOfYear(nYear);
			if (holiday != null) {
				holidays.add(holiday);
			}
		}

		// 念のためソートする
		Collections.sort(holidays);

		return holidays;
	}
}
