/*
 * HolidayListSerializer: 祝日リストシリアル化抽象基本クラス 
 */

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.Set;

import model.Serializer;

abstract class HolidayListSerializer extends Serializer
{
	//
	// 内部型定義
	//

	//
	// Factory:
	// Abstract FactoryパターンのAbstractFactoryクラス
	//
	abstract static protected class Factory
	{
		//
		// コンストラクタ
		//
		protected Factory()
		{
		}

		//
		// 再定義必須メソッド
		//
		abstract boolean canCreate(String strFileName);
		abstract HolidayListSerializer create(String strFileName, Set setHoliday);
	}

	//
	// クラス属性
	//

	private static final Factory[] factories;

	//
	// クラスコンストラクタ
	//
	static
	{
		factories = new Factory[4];
		int nIndex = 0;
		factories[nIndex++] = new ICalendarHolidayListSerializer.Factory();
		factories[nIndex++] = new HowmHolidayListSerializer.Factory();
		factories[nIndex++] = new QualendarHolidayListSerializer.Factory();
		factories[nIndex++] = new TextHolidayListSerializer.Factory();
	}

	//
	// クラスメソッド(公開)
	//
	
	// ファイル用インスタンスを返す
	static public HolidayListSerializer getInstanceForFile(
		String strFileName,	// ファイル名
		Set setHoliday)		// 休日リスト
	{
		if (strFileName == null) {
			throw new NullPointerException();
		}
		else if (strFileName.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (setHoliday == null) {
			throw new NullPointerException();
		}
		
		HolidayListSerializer serializer = null;
		for (int nIndex = 0; nIndex < factories.length; nIndex++) {
			if (factories[nIndex].canCreate(strFileName)) {
				serializer = factories[nIndex].create(strFileName, setHoliday);
				break; 
			}
		}
		if (serializer == null) {
			serializer = new TextHolidayListSerializer(setHoliday);
		}
 		
		return serializer;
	}
	
	//
	// 属性
	//
	private final Set holidays;

	//
	// コンストラクタ
	//
	protected HolidayListSerializer(
		Set setHoliday,			// 休日リスト
		String strCharsetName)	// 文字符号化方式の文字列表現
	{
		super(strCharsetName);

		holidays = setHoliday;
	}

	//
	// 再定義メソッド(実装)
	//

	// 出力する	
	final protected void writeBody(
		Writer writer)	// 出力ストリーム
		throws IOException
	{
		Iterator it = holidays.iterator();
		while (it.hasNext()) {
			writeRecord((HolidayInfo.DateOfYear)it.next(), writer);
		}
	}

	//
	// 再定義必須メソッド(実装)
	//

	abstract protected void writeRecord(HolidayInfo.DateOfYear holidayOfYear, 
											Writer writer)
											throws IOException; 
}
