/*
 * QualendarHolidayListSerializer: Qualendar祝日定義出力
 * http://ichitokumei.hp.infoseek.co.jp/qualendar/
 */

import java.io.IOException;
import java.io.Writer;
import java.util.Set;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

import environment.EnvInfo;

final public class QualendarHolidayListSerializer extends HolidayListSerializer
{
	//
	// 内部型定義
	//
	
	//
	// Factory:
	// Abstract FactoryパターンのConcreteFactoryクラス
	//
	final static class Factory extends HolidayListSerializer.Factory
	{
		//
		// コンストラクタ
		//
		Factory()
		{
			super();
		}
		
		//
		// 再定義メソッド
		//
		
		// 作成可能かを返す 
		boolean canCreate(
			String strFileName)	// ファイル名
		{
			if (strFileName == null) {
				throw new NullPointerException();
			}
			else if (strFileName.length() == 0) {
				throw new IllegalArgumentException();
			}

			return (strFileName.toLowerCase().endsWith(QualendarHolidayListSerializer.FILE_EXT));
		}
		
		// インスタンスを作成する
		HolidayListSerializer create(
			String strFileName,	// ファイル名
			Set setHoliday)		// 祝日定義の集合
		{
			if (strFileName == null) {
				throw new NullPointerException();
			}
			else if (strFileName.length() == 0) {
				throw new IllegalArgumentException();
			}
			if (setHoliday == null) {
				throw new NullPointerException();
			}
		
			return new QualendarHolidayListSerializer(setHoliday);
		}
	}

	//
	// 定数
	//
	
	static public final String FILE_EXT = ".qual.txt"; 

	static private final DateFormat dateFormat = new SimpleDateFormat("yyyy M d");

	//
	// コンストラクタ
	//
	private QualendarHolidayListSerializer(
		Set setHoliday)	// 祝日定義の集合
	{
		super(setHoliday, EnvInfo.getInstance().getDefaultEncoding());
	}

	//
	// 再定義メソッド(実装)
	//
	
	// ヘッダを書きこむ
	protected void writeHeader(
		Writer writer)	// 出力先
		throws IOException
	{
		// Do nothing.
	}

	// レコードを書きこむ
	protected void writeRecord(
		HolidayInfo.DateOfYear holidayOfYear,	// 
		Writer writer)							// 出力先
		throws IOException
	{
		writer.write(dateFormat.format(holidayOfYear.date()));
		writer.write(" ");
		writer.write(holidayOfYear.name());
		writer.write("\r\n");
	}

	// フッタを書きこむ
	protected void writeFooter(
		Writer writer)	// 出力先
		throws IOException
	{
		// Do nothing.
	}

}
