/*
 * SBHoliday: StompBox/Holiday アプリケーションメインクラス
 */

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.List;

import model.Serializer;
import util.Utilities;

// TODO 編集機能追加
public final class SBHoliday
{
	//
	// 定数定義
	//
	
	static final String OPTION_YEAR = "year";
	static final String OPTION_DIRECT = "direct";
	
	//
	// クラスメソッド(公開)
	//

	// エントリ
	public static void main(
		String[] args)	// 引数(入力)
	{
		try {
			HolidayInfoList holidays = loadHolidayInfo(); 
			ArgumentInfo argument = new ArgumentInfo(true, args);
			if (argument.hasOption(OPTION_DIRECT)) {
				boolean bSuccess = outputDirect(argument, holidays);
				System.exit(bSuccess != false ? 0 : 1);
			}
			else {
				MainFrame frame = MainFrame.getInstance();
				frame.setHolidayInfo(holidays);
				if (frame.initialize(argument)) {
					frame.show();
				}
				else {
					System.exit(1);
				}
			}
		}
		catch (Exception exp) {
			Utilities.reportException("StompBox/Holiday", exp);
			System.exit(1);
		}
	}

	//
	// クラスメソッド(実装)
	//

	// 直接出力する
	private static boolean outputDirect(
		ArgumentInfo argument,		// 引数情報
		HolidayInfoList holidays)	// 祝日定義
		throws IOException
	{
		boolean bSuccess = false;
		
		if (argument == null) {
			throw new NullPointerException();
		}
		if (holidays == null) {
			throw new NullPointerException();
		}
		
		String strFilePath = argument.getOptionValue(OPTION_DIRECT);
		int nYear = MainFrame.getYearFromArgument(argument);
		if (strFilePath != null && 0 < strFilePath.length()) {
			File file = new File(strFilePath);
			if (file.exists()) {
				System.err.print(strFilePath + " is already exist; Stop output.");
				return false;
			}

			Serializer serializer = HolidayListSerializer.getInstanceForFile(strFilePath,
																			MainFrame.getHolidaysOfYear(holidays, nYear));
			serializer.serialize(new FileOutputStream(strFilePath));
			bSuccess = true;
		}
		else {
			List list = MainFrame.getHolidaysText(holidays, nYear);
			Iterator it = list.iterator();
			while (it.hasNext()) {
				System.out.println(it.next().toString());
			}
			bSuccess = true;
		}
		return bSuccess;
	}

	// 祝日情報を読みこむ
	private static HolidayInfoList loadHolidayInfo()
		throws IOException
	{
		InputStream streamHolidayInfo = SBHoliday.class.getResourceAsStream("/holiday_JP.ja.csv");
		return HolidayInfoList.createFromFile(streamHolidayInfo);
	}
}
