/*
 * TextHolidayListSerializer: テキストフォーマット出力 
 */

import java.io.IOException;
import java.io.Writer;
import java.util.Set;

import environment.EnvInfo;

final class TextHolidayListSerializer extends HolidayListSerializer
{
	//
	// 内部型定義
	//
	
	//
	// Factory:
	// Abstract FactoryパターンのConcreteFactoryクラス
	//
	final static class Factory extends HolidayListSerializer.Factory
	{
		//
		// コンストラクタ
		//
		Factory()
		{
			super();
		}
		
		//
		// 再定義メソッド
		//
		
		// 作成可能かを返す 
		boolean canCreate(
			String strFileName)	// ファイル名
		{
			if (strFileName == null) {
				throw new NullPointerException();
			}
			else if (strFileName.length() == 0) {
				throw new IllegalArgumentException();
			}

			// 常に作成可能
			return true;
		}
		
		// インスタンスを作成する
		HolidayListSerializer create(
			String strFileName,	// ファイル名
			Set setHoliday)		// 祝日定義の集合
		{
			if (strFileName == null) {
				throw new NullPointerException();
			}
			else if (strFileName.length() == 0) {
				throw new IllegalArgumentException();
			}
			if (setHoliday == null) {
				throw new NullPointerException();
			}
		
			return new TextHolidayListSerializer(setHoliday);
		}
	}

	//
	// コンストラクタ
	//
	TextHolidayListSerializer(
		Set setHoliday)	// 祝日定義の集合
	{
		super(setHoliday, EnvInfo.getInstance().getDefaultEncoding());
	}

	//
	// 再定義メソッド(実装)
	//
	
	// ヘッダを書きこむ
	protected void writeHeader(
		Writer writer)	// 出力先
		throws IOException
	{
		// Do nothing.
	}

	// レコードを書きこむ
	protected void writeRecord(
		HolidayInfo.DateOfYear holidayOfYear,	// 
		Writer writer)							// 出力先
		throws IOException
	{
		writer.write(MainFrame.formatHoliday(holidayOfYear));
		writer.write(EnvInfo.NEWLINE);
	}

	// フッタを書きこむ
	protected void writeFooter(
		Writer writer)	// 出力先
		throws IOException
	{
		// Do nothing.
	}

}
