/*
 * SetYearDialog : 年指定ダイアログ
 */

package view.awt;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import ui.awt.DialogUtilities;

import MainFrame;
import view.awt.SetYearDialogImp;

public final class SetYearDialog extends WindowAdapter
{
	//
	// 内部型定義
	//

	// OKAction : OKボタン動作
	private final class OKAction implements ActionListener
	{
		public void actionPerformed(
			ActionEvent e)	// イベント情報(入力)
		{
			closeByOk = false;
			try {
				int nYear = getOperation().getYear();
				if (0 < nYear) {
					closeByOk = true;
					dialogImp.hide();
				}
			}
			catch (NumberFormatException exp) {
				// yearField入力フィールドにはkeyTypedメソッドを定義して
				// 数字以外の入力を制限しているのだがSL-C860だと働かない。
				// 結果として変換の失敗例外が発生することがあるので
				// ここで捕まえて下の通常の処理に流れを戻す。
			}
			if (closeByOk == false) {
				// SL-C860のJ2MEにはDialogクラスにDialogを引数にした構築子がない
				// フレームを親にしても効果はほとんど変わらないのでそうする
			//	MainFrame.getInstance().showMessage(dialogImpl, "入力は年として解釈できません。");
				MainFrame.getInstance().showMessage("入力は年として解釈できません。");
				getOperation().windowActivated();
			}
		}
	}

	// CancelAction : 取り消しボタン動作
	private final class CancelAction implements ActionListener
	{
		public void actionPerformed(
			ActionEvent e)	// イベント情報(入力)
		{
			closeByOk = false;
			dialogImp.hide();
		}
	}
	//
	// 属性
	//
	private Dialog dialogImp;
	private boolean	closeByOk = false;

	//
	// 構築子
	//
	public SetYearDialog(
		Frame owner)	// 親フレーム(入力)
	{
		super();

		dialogImp = new view.awt.SetYearDialogImp(owner);
		dialogImp.addWindowListener(this);

		// タイトルを設定する
	//	dialogImpl.setTitle(MainFrame.getWindowTitle(getClass().getName()));
		dialogImp.setTitle(MainFrame.getFrameTitle("SetYearDialog"));
		dialogImp.setLayout(new FlowLayout(FlowLayout.LEFT));

		dialogImp.pack();
		dialogImp.setResizable(false);

		DialogUtilities.moveWindowToCenter(owner, dialogImp);

		KeyListener keyListenre = DialogUtilities.createNumberKeyListener(new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				getOperation().updateStatus();
			}
		});
		getOperation().setListener(keyListenre,
									new OKAction(),
									new CancelAction());
	}

	//
	// 状態
	//

	// OKボタンで閉じられたかを返す
	public boolean isCloseByOk()
	{
		return closeByOk;
	}

	// 指定年を返す
	public int getYear()
	{
		return getOperation().getYear();
	}

	//
	// 操作
	//

	/// ダイアログを表示する
	public void show()
	{
		dialogImp.show();
	}

	/// ダイアログを破棄する
	public void dispose()
	{
		dialogImp.dispose();
	}

	//
	// 実装
	//

	// ダイアログをインターフェースとして返す
	private SetYearDialogImp getOperation()
	{
		return (SetYearDialogImp)dialogImp;
	}
	
	//
	// 再定義メソッド(WindowAdapter)
	//

	// ウィンドウをアクティブ化する
	public void windowActivated(
		WindowEvent e)	// イベント情報(入力)
	{
		getOperation().windowActivated();
	}

	// ウィンドウを閉じる
	public void windowClosing(
		WindowEvent we)	// イベント情報(入力)
	{
		dialogImp.hide();
	}
}
