/*
 * MainFrameImp : Swing版メインウィンドウ
 */
package view.swing;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ItemListener;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.filechooser.FileFilter;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;

import environment.EnvInfo;
import io.FileUtilities;
import view.LabelManager;
import ui.swing.LookAndFeelUtilities;

import view.ImplementOperation;
import ICalendarHolidayListSerializer;
import QualendarHolidayListSerializer;
import HowmHolidayListSerializer;

public final class MainFrameImp extends JFrame
	implements ImplementOperation
{
	//
	// 内部型定義
	//
	
	// テキストファイルフィルタ
	static private class TextFileFilter extends FileFilter
	{
		//
		// 状態
		//

		// 受け入れ可能ファイルかを返す
		public boolean accept(
			File file)	// 判定対象ファイル 
		{
			return (file.isDirectory()
				|| file.getName().toLowerCase().endsWith(".txt"));
		}

		// 記述を返す 		
		public String getDescription()
		{
			return makeFileFilterDescription(LabelManager.LABEL_SAVE_TEXT,
												".txt"); 
		}
	}
	
	// iCalendarファイルフィルタ
	static private class ICalendarFileFilter extends FileFilter
	{
		//
		// 状態
		//

		// 受け入れ可能ファイルかを返す
		public boolean accept(
			File file)	// 判定対象ファイル 
		{
			return (file.isDirectory()
				|| file.getName().toLowerCase().endsWith(ICalendarHolidayListSerializer.FILE_EXT_ICS));
		}

		// 記述を返す 		
		public String getDescription()
		{
			return makeFileFilterDescription(LabelManager.LABEL_SAVE_ICALENDAR,
												ICalendarHolidayListSerializer.FILE_EXT_ICS); 
		}
	}
	
	// vCalendarファイルフィルタ
	static private class VCalendarFileFilter extends FileFilter
	{
		//
		// 状態
		//

		// 受け入れ可能ファイルかを返す
		public boolean accept(
			File file)	// 判定対象ファイル 
		{
			return (file.isDirectory()
				|| file.getName().toLowerCase().endsWith(ICalendarHolidayListSerializer.FILE_EXT_VCS));
		}

		// 記述を返す 		
		public String getDescription()
		{
			return makeFileFilterDescription(LabelManager.LABEL_SAVE_VCALENDAR,
												ICalendarHolidayListSerializer.FILE_EXT_VCS); 
		}
	}

	// Qualendarファイルフィルタ
	static private class QualendarFileFilter extends FileFilter
	{
		//
		// 状態
		//

		// 受け入れ可能ファイルかを返す
		public boolean accept(
			File file)	// 判定対象ファイル 
		{
			return (file.isDirectory()
				|| file.getName().toLowerCase().endsWith(QualendarHolidayListSerializer.FILE_EXT));
		}

		// 記述を返す 		
		public String getDescription()
		{
			return makeFileFilterDescription(LabelManager.LABEL_SAVE_QUALENDAR,
											QualendarHolidayListSerializer.FILE_EXT); 
		}
	}

	// howmファイルフィルタ
	static private class HowmFileFilter extends FileFilter
	{
		//
		// 状態
		//

		// 受け入れ可能ファイルかを返す
		public boolean accept(
			File file)	// 判定対象ファイル 
		{
			return (file.isDirectory()
				|| file.getName().toLowerCase().endsWith(HowmHolidayListSerializer.FILE_EXT));
		}

		// 記述を返す 		
		public String getDescription()
		{
			return makeFileFilterDescription(LabelManager.LABEL_SAVE_HOWM,
											HowmHolidayListSerializer.FILE_EXT); 
		}
	}

	//
	// クラスメソッド
	//
		
	// ファイルフィルタ用説明文を作成する
	static String makeFileFilterDescription(
		String strKeyName,		// キー名
		String strExtension)	// 拡張子
	{
		if (strKeyName == null) {
			throw new NullPointerException();
		}
		else if (strKeyName.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (strExtension == null) {
			throw new NullPointerException();
		}
		else if (strExtension.length() == 0) {
			throw new IllegalArgumentException();
		}

		return FileUtilities.makeFileFilterDescription(LabelManager.getLabelProperties().getProperty(strKeyName),
														strExtension);
	}

	//
	// 属性
	//
	private List lafMenuItems;
	private JComboBox	yearChoice;
	private JButton	saveButton;
	private JList	holidayList;

	private SetYearDialog setYearDialog;

	//
	// 構築子
	//
	public MainFrameImp()
	{
		super();
	}

	//
	// 実装
	//

	// メニューを作成する
	private void initMenu()
	{
		List list = LookAndFeelUtilities.addSelectMenuItem(this);
		if (list.size() < 2) {
			return;
		}

		JMenuBar menuBar = new JMenuBar();

		Font objMenuFont = environment.EnvInfo.getInstance().getMenuFont(this);
		if (objMenuFont != null) {
			menuBar.setFont((Font)objMenuFont);
		}

		Properties properties = LabelManager.getLabelProperties();

		// 「表示」メニュー
		lafMenuItems = list;

		JMenu menuView = new JMenu(properties.getProperty(LabelManager.MENU_VIEW));
		menuView.setMnemonic(KeyEvent.VK_V);
		Iterator it = lafMenuItems.iterator();
		while (it.hasNext()) {
			JRadioButtonMenuItem item = (JRadioButtonMenuItem)it.next();
			menuView.add(item);
		}
		menuBar.add(menuView);

		setJMenuBar(menuBar);
	}


	//
	// 再定義メソッド(MainFrameBaseImplementor)
	//

	// コンポーネントを初期化する
	public void initComponent()	// コンポーネント用リソース
	{
		Container contentPane = getContentPane();

		// フォントを設定する
		contentPane.setFont(EnvInfo.getInstance().getBaseFont(this));

		initMenu();

		// 年選択コンボボックスを作成する
		yearChoice = new JComboBox();

		// 保存ボタンを作成する
		saveButton = new JButton("保存");

		// ツールバー風パネルを作成する
		JPanel toolBarContainer = new JPanel();
		toolBarContainer.setLayout(new FlowLayout(FlowLayout.LEFT));
		toolBarContainer.add(new JLabel("西暦"));
		toolBarContainer.add(yearChoice);
		toolBarContainer.add(new JLabel("年の祝祭日"));
		toolBarContainer.add(saveButton);

		// 祝日表示用リストを設定する
		holidayList = new JList();

		// メインウィンドウに各コントロールを配置する
		contentPane.setLayout(new BorderLayout());
		contentPane.add(toolBarContainer, BorderLayout.NORTH);
		contentPane.add(new JScrollPane(holidayList), BorderLayout.CENTER);
	}

	// メッセージダイアログを表示する
	public void showMessage(
		String strMessage)	// メッセージ文字列(入力)
	{
		getToolkit().beep();
		JOptionPane.showMessageDialog(this, strMessage, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	//
	// 再定義メソッド(ImplementOperation)
	//

	// 選択されている年を文字列のまま返す
	public String getSelectedYearValue()
	{
		return (String)yearChoice.getModel().getSelectedItem();
	}

	// 祝日テキスト保存ファイル名を取得して返す
	public String getSaveFileFullPath(
		String strDialogTitle,	// ダイアログのタイトル
		String strDefPath,		// デフォルトの保存パス
		String strDefFileName)	// デフォルトのファイル名
	{
		String strSavePath = null;
		JFileChooser fileChooser = new JFileChooser(new File(strDefPath));
		fileChooser.setDialogTitle(strDialogTitle);
	 	FileFilter defaultFilter = new TextFileFilter();
		fileChooser.addChoosableFileFilter(new ICalendarFileFilter());
		fileChooser.addChoosableFileFilter(new VCalendarFileFilter());
		fileChooser.addChoosableFileFilter(defaultFilter);
		fileChooser.addChoosableFileFilter(new HowmFileFilter());
		fileChooser.addChoosableFileFilter(new QualendarFileFilter());
		fileChooser.setFileFilter(defaultFilter);
		fileChooser.setSelectedFile(new File(strDefFileName));
		while (strSavePath == null) {
			if (fileChooser.showSaveDialog(this) != JFileChooser.APPROVE_OPTION) {
				break;
			}
			strSavePath = fileChooser.getSelectedFile().getPath();
			if (new File(strSavePath).exists()) {
				String strMsg = strSavePath + "は存在します。上書きしますか?";
				int nResult = JOptionPane.showConfirmDialog(this,
															strMsg,
															"確認",
															JOptionPane.YES_NO_OPTION,
															JOptionPane.QUESTION_MESSAGE);
				if (nResult != JOptionPane.YES_OPTION) {
					strSavePath = null;
				}
			}
		}
		return strSavePath;
	}

	// 年選択イベントリスナーを設定する
	public void setYearChoiceListener(
		ItemListener listener)	// (入力)
	{
		if(listener == null) {
			throw new IllegalArgumentException();
		}
		
		yearChoice.addItemListener(listener);
	}

	// 保存イベントリスナーを設定する
	public void setSaveListener(
		ActionListener listener)	// (入力)
	{
		saveButton.addActionListener(listener);
	}

	// 年選択リストを設定する
	public void setYearList(
		java.util.List yearList,	// 選択年リスト(入力)
		int nYear)					// 選択年(入力)
	{
		DefaultComboBoxModel model = new DefaultComboBoxModel(yearList.toArray()); 
		model.setSelectedItem(Integer.toString(nYear));
		yearChoice.setModel(model);
	}

	// 年を設定する
	public void setYear(
		int nSetYear)	// 設定年(入力)
	{
		String setYearString = Integer.toString(nSetYear);

		DefaultComboBoxModel model = (DefaultComboBoxModel)yearChoice.getModel(); 
		int itemCount = yearChoice.getItemCount();
		int index = 1;
		while (index < itemCount) {
			int year = Integer.parseInt((String)model.getElementAt(index));
			if (year == nSetYear) {
				break;
			}
			else if (nSetYear < year) {
				model.insertElementAt(setYearString, index);
				break;
			}
			index++;
		}
		if (index == itemCount) {
			model.addElement(setYearString);
		}
		yearChoice.setSelectedItem(setYearString);
	}

	// 祝日リストを設定する
	public void setHolidayList(
		java.util.List holidayTextList)	// 祝日文字列リスト(入力)
	{
 		DefaultListModel newModel = new DefaultListModel();
		Iterator it = holidayTextList.iterator();
		while (it.hasNext()) {
			newModel.addElement(it.next());
		}
 		holidayList.setModel(newModel);
		if (0 < newModel.size()) {
			holidayList.ensureIndexIsVisible(0);
		}
	}

	// 年指定ダイアログを表示する
	// 戻り値: 入力された年(-1のときキャンセル)
	public int showSetYearDialog(
		int nCurrentYear)	// 現在年
	{
		if (setYearDialog == null) {
			setYearDialog = new SetYearDialog(this);
		}
		setYearDialog.show();
		return (setYearDialog.isCloseByOk()
				? setYearDialog.getYear()
				: -1);
	}
	
	// 終了準備処理を行う
	public void prepareExit()
	{
		if (setYearDialog != null) {
			setYearDialog.dispose();
		}
	}

	// 追加の状態を返す
	public Map getAdditionalState()
	{
		Map map = null;
		if (lafMenuItems != null) {
			map = LookAndFeelUtilities.getMenuItemState(lafMenuItems); 
		}
		return map;
	}

	// 追加の状態を設定する
	public void setAdditionalState(
		Map properties)	// 状態プロパティ
	{
		if (lafMenuItems != null) {
			LookAndFeelUtilities.setMenuItemState(properties, lafMenuItems);
		}
	}
}
