/*
 * SetYearDialog : 年指定ダイアログボックス
 */

package view.swing;

import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

import MainFrame;

final class SetYearDialog extends JDialog
	implements KeyListener
{
	//
	// クラス属性
	//
	private static final JLabel label;

	//
	// クラス初期化子
	//
	static
	{
		label = new JLabel("年:");
	}

	//
	// 属性
	//
	private JTextField	yearField;
	private JButton	OkButton;
	private JButton	cancelButton;

	private boolean	closeByOk = false;

	//
	// 構築子
	//
	SetYearDialog(
		JFrame owner)	// 親となるフレーム(入力)
	{
		// モーダルにするために第二引数にtrueを指定
		super(owner, true);

		Container contentPane = getContentPane();
		contentPane.add(label);

		yearField = new JTextField(4);
		yearField.addKeyListener(this);
		contentPane.add(yearField);

		OkButton = new JButton(" OK ");
		OkButton.setEnabled(false);
		OkButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e)	// イベント情報(入力)
			{
				closeByOk = false;
				try {
					int nYear = getYear();
					if (0 < nYear) {
						closeByOk = true;
						dispose();
					}
				}
				catch (NumberFormatException exp) {
					// yearField入力フィールドにはkeyTypedメソッドを定義して
					// 数字以外の入力を制限しているのだがSL-C860だと働かない。
					// 結果として変換の失敗例外が発生することがあるので
					// ここで捕まえて下の通常の処理に流れを戻す。
				}
				if (closeByOk == false) {
				//	MainFrame.getInstance().showMessage(dialogImpl, "入力は年として解釈できません。");
					MainFrame.getInstance().showMessage("入力は年として解釈できません。");
				//	windowActivated();
				}
			}
		});
		contentPane.add(OkButton);

		cancelButton = new JButton("取消");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e)	// イベント情報(入力)
			{
				closeByOk = false;
				dispose();
			}
		});
		contentPane.add(cancelButton);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				SetYearDialog.this.dispose();
			}

			public void windowActivated() {
				OkButton.setEnabled(0 < yearField.getText().length());
				yearField.requestFocus();
			}
		});

		setTitle(MainFrame.getFrameTitle("SetYearDialog"));
		contentPane.setLayout(new FlowLayout(FlowLayout.LEFT));

		pack();
		setResizable(false);

		setLocationRelativeTo(owner);
	}

	//
	// 状態
	//

	// OKボタンで閉じられたかを返す
	public boolean isCloseByOk()
	{
		return closeByOk;
	}

	// 指定年を返す
	int getYear()
	{
		return Integer.parseInt(yearField.getText());
	}

	//
	// 実装
	//
	
	// 状態を更新する
	private void updateStatus()
	{
		OkButton.setEnabled(0 < yearField.getText().length());
	}

	//
	// 再定義メソッド(KeyListener)
	//

	// キータイプ処理を行う
	public void keyTyped(
		KeyEvent e)	// イベント情報(入力)
	{
		// 入力を数字のみに制限する
		if (Character.isISOControl(e.getKeyChar()) == false
		&& Character.isDigit(e.getKeyChar()) == false) {
			e.consume();
			getToolkit().beep();
		}
	}

	// キー押し下げ処理を行う
	public void keyPressed(
		KeyEvent e)	// イベント情報(入力)
	{
		// Do nothing.
	}

	// キーリリース処理を行う
	public void keyReleased(
		KeyEvent e)	// イベント情報(入力)
	{
		updateStatus();
	}
}
